@@ -21,7 +21,6 @@ module Crypto.Random.AESCtr
     ) where
 
 import Crypto.Random
-import System.Random (RandomGen(..))
 import Crypto.Random.AESCtr.Internal
 import Control.Arrow (second)
 
@@ -42,10 +41,10 @@ import Data.Bits (xor, (.&.))
 --
 -- By default, this generator will automatically reseed after generating
 -- 1 megabyte of data.
-data AESRNG = AESRNG { aesrngState     :: RNG
+data AESRNG = AESRNG { aesrngState     :: !RNG
                      , aesrngEntropy   :: EntropyPool
-                     , aesrngThreshold :: Int -- ^ in number of generated block
-                     , aesrngCache     :: ByteString }
+                     , aesrngThreshold :: !Int -- ^ in number of generated block
+                     , aesrngCache     :: !ByteString }
 
 instance Show AESRNG where
     show _ = "aesrng[..]"
@@ -56,8 +55,7 @@ makeFrom entPool b = AESRNG
     , aesrngEntropy      = entPool
     , aesrngThreshold    = 1024 -- in blocks generated, so 1mb
     , aesrngCache        = B.empty }
-  where rng            = RNG (get128 iv) (get128 cnt) 0 key
-        (key, cnt, iv) = makeParams b
+  where rng = makeRNG b
 
 -- | make an AES RNG from an EntropyPool.
 --
@@ -100,15 +98,10 @@ genRanBytes rng n = second reseedThreshold $ genRanBytesNoCheck rng n
 reseedThreshold :: AESRNG -> AESRNG
 reseedThreshold rng
     | getNbChunksGenerated (aesrngState rng) >= lvl =
-         let ent = toBytes $ grabEntropy 64 (aesrngEntropy rng)
-          in rng { aesrngState = reseedState ent (aesrngState rng) }
+         let newRngState = makeRNG $ toBytes $ grabEntropy 64 (aesrngEntropy rng)
+          in rng { aesrngState = newRngState }
     | otherwise  = rng
-  where
-        lvl = aesrngThreshold rng
-        reseedState :: ByteString -> RNG -> RNG
-        reseedState b g@(RNG _ cnt1 _ _) = RNG (get128 r16 `xor128` get128 iv2) (cnt1 `xor128` get128 cnt2) 0 key2
-            where (r16, _)          = genNextChunk g
-                  (key2, cnt2, iv2) = makeParams b
+  where lvl = aesrngThreshold rng
 
 instance CPRG AESRNG where
     cprgCreate                      = make
@@ -122,6 +115,7 @@ instance CPRG AESRNG where
     cprgFork rng = let (b,rng') = genRanBytes rng 64
                     in (rng', makeFrom (aesrngEntropy rng) b)
 
+{-
 instance RandomGen AESRNG where
     next rng =
         let (bs, rng') = genRanBytes rng 16 in
@@ -132,3 +126,4 @@ instance RandomGen AESRNG where
         let rng' = make (aesrngEntropy rng)
          in (rng, rng')
     genRange _ = (0, 0x7fffffff)
+-}
